@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2005-2008 Team XBMC
+ *      Copyright (C) 2005-2012 Team XBMC
  *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
@@ -13,23 +13,22 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
- *  http://www.gnu.org/copyleft/gpl.html
+ *  along with XBMC; see the file COPYING.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "DVDInputStreamNavigator.h"
-#include "Util.h"
-#include "LangCodeExpander.h"
-#include "DVDDemuxSPU.h"
+#include "utils/LangCodeExpander.h"
+#include "../DVDDemuxSPU.h"
 #include "DVDStateSerializer.h"
-#include "GUISettings.h"
+#include "settings/GUISettings.h"
 #include "LangInfo.h"
 #include "utils/log.h"
-#include "Geometry.h"
-#ifdef __APPLE__
-#include "CocoaInterface.h"
+#include "guilib/Geometry.h"
+#include "utils/URIUtils.h"
+#if defined(TARGET_DARWIN)
+#include "osx/CocoaInterface.h"
 #endif
 
 #define HOLDMODE_NONE 0
@@ -51,6 +50,11 @@ CDVDInputStreamNavigator::CDVDInputStreamNavigator(IDVDPlayer* player) : CDVDInp
   m_iTitle = m_iTitleCount = 0;
   m_iPart = m_iPartCount = 0;
   m_iTime = m_iTotalTime = 0;
+  m_bEOF = false;
+  m_icurrentGroupId = 0;
+  m_lastevent = DVDNAV_NOP;
+
+  memset(m_lastblock, 0, sizeof(m_lastblock));
 }
 
 CDVDInputStreamNavigator::~CDVDInputStreamNavigator()
@@ -60,11 +64,10 @@ CDVDInputStreamNavigator::~CDVDInputStreamNavigator()
 
 bool CDVDInputStreamNavigator::Open(const char* strFile, const std::string& content)
 {
-  char* strDVDFile;
   m_icurrentGroupId = 0;
-  if (!CDVDInputStream::Open(strFile, "video/x-dvd-mpeg")) 
+  if (!CDVDInputStream::Open(strFile, "video/x-dvd-mpeg"))
     return false;
-  
+
   // load libdvdnav.dll
   if (!m_dll.Load())
     return false;
@@ -72,33 +75,32 @@ bool CDVDInputStreamNavigator::Open(const char* strFile, const std::string& cont
   // load the dvd language codes
   // g_LangCodeExpander.LoadStandardCodes();
 
-  // since libdvdnav automaticly play's the dvd if the directory contains VIDEO_TS.IFO
-  // we strip it here.
-  strDVDFile = strdup(strFile);
-#ifndef _LINUX
-  if (strnicmp(strDVDFile + strlen(strDVDFile) - 12, "VIDEO_TS.IFO", 12) == 0)
-#else
-  if (strncasecmp(strDVDFile + strlen(strDVDFile) - 12, "VIDEO_TS.IFO", 12) == 0)
-#endif
-  {
-    strDVDFile[strlen(strDVDFile) - 13] = '\0';
-  }
-#if defined(__APPLE__)
+  // libdvdcss fails if the file path contains VIDEO_TS.IFO or VIDEO_TS/VIDEO_TS.IFO
+  // libdvdnav is still able to play without, so strip them.
+
+  CStdString path = strFile;
+  if(URIUtils::GetFileName(path) == "VIDEO_TS.IFO")
+    path = URIUtils::GetParentPath(path);
+  URIUtils::RemoveSlashAtEnd(path);
+  if(URIUtils::GetFileName(path) == "VIDEO_TS")
+    path = URIUtils::GetParentPath(path);
+  URIUtils::RemoveSlashAtEnd(path);
+
+#if defined(TARGET_DARWIN_OSX)
   // if physical DVDs, libdvdnav wants "/dev/rdiskN" device name for OSX,
   // strDVDFile will get realloc'ed and replaced IF this is a physical DVD.
-  Cocoa_MountPoint2DeviceName(strDVDFile);
+  char* strDVDFile = Cocoa_MountPoint2DeviceName(strdup(path.c_str()));
+  path = strDVDFile;
+  free(strDVDFile);
 #endif
 
   // open up the DVD device
-  if (m_dll.dvdnav_open(&m_dvdnav, strDVDFile) != DVDNAV_STATUS_OK)
+  if (m_dll.dvdnav_open(&m_dvdnav, path.c_str()) != DVDNAV_STATUS_OK)
   {
-    free(strDVDFile);
-
     CLog::Log(LOGERROR,"Error on dvdnav_open\n");
     Close();
     return false;
   }
-  free(strDVDFile);
 
   int region = g_guiSettings.GetInt("dvds.playerregion");
   int mask = 0;
@@ -243,21 +245,12 @@ int CDVDInputStreamNavigator::Read(BYTE* buf, int buf_size)
 }
 
 // not working yet, but it is the recommanded way for seeking
-__int64 CDVDInputStreamNavigator::Seek(__int64 offset, int whence)
+int64_t CDVDInputStreamNavigator::Seek(int64_t offset, int whence)
 {
-  if (!m_dvdnav) return -1;
-  uint32_t pos=0, len=1;
-  if (m_dll.dvdnav_sector_search(m_dvdnav, (uint64_t)(offset / DVD_VIDEO_LB_LEN), (int32_t)whence) != DVDNAV_STATUS_ERR)
-  {
-    m_dll.dvdnav_get_position(m_dvdnav, &pos, &len);
-  }
+  if(whence == SEEK_POSSIBLE)
+    return 0;
   else
-  {
-    CLog::Log(LOGDEBUG, "dvdnav: %s", m_dll.dvdnav_err_to_string(m_dvdnav));
     return -1;
-  }
-
-  return (int)(pos * DVD_VIDEO_LB_LEN);
 }
 
 int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
@@ -265,7 +258,7 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
   if (!m_dvdnav) return -1;
 
   int result;
-  int len;
+  int len = 2048;
   int iNavresult = NAVRESULT_NOP;
 
   // m_tempbuffer will be used for anything that isn't a normal data block
@@ -275,28 +268,14 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
   if(m_holdmode == HOLDMODE_HELD)
     return NAVRESULT_HOLD;
 
-  try
-  {
-    // the main reading function
-    if(m_holdmode == HOLDMODE_SKIP)
-    { /* we where holding data, return the data held */
-      m_holdmode = HOLDMODE_DATA;
-      result = DVDNAV_STATUS_OK;
-    }
-    else
-      result = m_dll.dvdnav_get_next_cache_block(m_dvdnav, &buf, &m_lastevent, &len);
-
-  }
-  catch (...)
-  {
-    CLog::Log(LOGERROR, "CDVDInputStreamNavigator::ProcessBlock - exception thrown in dvdnav_get_next_cache_block.");
-
-    // okey, we are probably holding a vm_lock here so leave it.. this could potentialy cause problems if we aren't holding it
-    // but it's more likely that we do
-    LeaveCriticalSection((LPCRITICAL_SECTION)&(m_dvdnav->vm_lock));
-    m_bEOF = true;
-    return NAVRESULT_ERROR;
+  // the main reading function
+  if(m_holdmode == HOLDMODE_SKIP)
+  { /* we where holding data, return the data held */
+    m_holdmode = HOLDMODE_DATA;
+    result = DVDNAV_STATUS_OK;
   }
+  else
+    result = m_dll.dvdnav_get_next_cache_block(m_dvdnav, &buf, &m_lastevent, &len);
 
   if (result == DVDNAV_STATUS_ERR)
   {
@@ -329,6 +308,7 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
         // user input to make menus and other interactive stills work.
         // A length of 0xff means an indefinite still which has to be skipped
         // indirectly by some user interaction.
+        m_holdmode = HOLDMODE_NONE;
         iNavresult = m_pDVDPlayer->OnDVDNavResult(buf, DVDNAV_STILL_FRAME);
 
         /* if user didn't care for action, just skip it */
@@ -376,6 +356,12 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
         //libdvdnav never sets logical, why.. don't know..
         event->logical = GetActiveSubtitleStream();
 
+        if(event->logical<0 && GetSubTitleStreamCount()>0)
+        {
+          /* this will not take effect in this event */
+          CLog::Log(LOGINFO, "%s - none or invalid subtitle stream selected, defaulting to first", __FUNCTION__);
+          SetActiveSubtitleStream(0);
+        }
         m_bCheckButtons = true;
         iNavresult = m_pDVDPlayer->OnDVDNavResult(buf, DVDNAV_SPU_STREAM_CHANGE);
       }
@@ -427,9 +413,10 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
           iNavresult = NAVRESULT_HOLD;
         }
         else
+        {
+          m_bInMenu   = (0 == m_dll.dvdnav_is_domain_vts(m_dvdnav));
           iNavresult = m_pDVDPlayer->OnDVDNavResult(buf, DVDNAV_VTS_CHANGE);
-
-        m_bInMenu = (0 == m_dll.dvdnav_is_domain_vts(m_dvdnav));
+        }
       }
       break;
 
@@ -466,6 +453,7 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
 
         dvdnav_cell_change_event_t* cell_change_event = (dvdnav_cell_change_event_t*)buf;
         m_iCellStart = cell_change_event->cell_start; // store cell time as we need that for time later
+        m_iTime      = (int) (m_iCellStart / 90);
         m_iTotalTime = (int) (cell_change_event->pgc_length / 90);
         m_icurrentGroupId = cell_change_event->pgN * 1000 + cell_change_event->cellN;
 
@@ -489,7 +477,7 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
         // Calculate current time
         //unsigned int pos, len;
         //m_dll.dvdnav_get_position(m_dvdnav, &pos, &len);
-        //m_iTime = (int)(((__int64)m_iTotalTime * pos) / len);
+        //m_iTime = (int)(((int64_t)m_iTotalTime * pos) / len);
 
         pci_t* pci = m_dll.dvdnav_get_current_nav_pci(m_dvdnav);
         m_dll.dvdnav_get_current_nav_dsi(m_dvdnav);
@@ -504,7 +492,7 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
         m_bInMenu = pci->hli.hl_gi.hli_ss || (0 == m_dll.dvdnav_is_domain_vts(m_dvdnav));
 
         /* check for any gap in the stream, this is likely a discontinuity */
-        __int64 gap = (__int64)pci->pci_gi.vobu_s_ptm - m_iVobUnitStop;
+        int64_t gap = (int64_t)pci->pci_gi.vobu_s_ptm - m_iVobUnitStop;
         if(gap)
         {
           /* make sure demuxer is flushed before we change any correction */
@@ -517,7 +505,7 @@ int CDVDInputStreamNavigator::ProcessBlock(BYTE* dest_buffer, int* read)
           }
           m_iVobUnitCorrection += gap;
 
-          CLog::Log(LOGDEBUG, "DVDNAV_NAV_PACKET - DISCONTINUITY FROM:%"PRId64" TO:%"PRId64" DIFF:%"PRId64, (m_iVobUnitStop * 1000)/90, ((__int64)pci->pci_gi.vobu_s_ptm*1000)/90, (gap*1000)/90);
+          CLog::Log(LOGDEBUG, "DVDNAV_NAV_PACKET - DISCONTINUITY FROM:%"PRId64" TO:%"PRId64" DIFF:%"PRId64, (m_iVobUnitStop * 1000)/90, ((int64_t)pci->pci_gi.vobu_s_ptm*1000)/90, (gap*1000)/90);
         }
 
         m_iVobUnitStart = pci->pci_gi.vobu_s_ptm;
@@ -765,7 +753,7 @@ void CDVDInputStreamNavigator::OnBack()
 // we don't allow skipping in menu's cause it will remove menu overlays
 void CDVDInputStreamNavigator::OnNext()
 {
-  if (m_dvdnav && !IsInMenu())
+  if (m_dvdnav && !(IsInMenu() && GetTotalButtons() > 0))
   {
     m_dll.dvdnav_next_pg_search(m_dvdnav);
   }
@@ -774,7 +762,7 @@ void CDVDInputStreamNavigator::OnNext()
 // we don't allow skipping in menu's cause it will remove menu overlays
 void CDVDInputStreamNavigator::OnPrevious()
 {
-  if (m_dvdnav && !IsInMenu())
+  if (m_dvdnav && !(IsInMenu() && GetTotalButtons() > 0))
   {
     m_dll.dvdnav_prev_pg_search(m_dvdnav);
   }
@@ -792,15 +780,17 @@ void CDVDInputStreamNavigator::SkipWait()
   m_dll.dvdnav_wait_skip(m_dvdnav);
 }
 
-void CDVDInputStreamNavigator::SkipHold()
+CDVDInputStream::ENextStream CDVDInputStreamNavigator::NextStream()
 {
-  if(IsHeld())
+  if(m_holdmode == HOLDMODE_HELD)
     m_holdmode = HOLDMODE_SKIP;
-}
 
-bool CDVDInputStreamNavigator::IsHeld()
-{
-  return m_holdmode == HOLDMODE_HELD;
+  if(m_bEOF)
+    return NEXTSTREAM_NONE;
+  else if(m_lastevent == DVDNAV_VTS_CHANGE)
+    return NEXTSTREAM_OPEN;
+  else
+    return NEXTSTREAM_RETRY;
 }
 
 int CDVDInputStreamNavigator::GetActiveSubtitleStream()
@@ -1055,12 +1045,48 @@ bool CDVDInputStreamNavigator::SeekTime(int iTimeInMsec)
 
 bool CDVDInputStreamNavigator::SeekChapter(int iChapter)
 {
+  if (!m_dvdnav)
+    return false;
+
+  // cannot allow to return true in case of buttons (overlays) because otherwise back in DVDPlayer FlushBuffers will remove menu overlays
+  // therefore we just skip the request in case there are buttons and return false
+  if (IsInMenu() && GetTotalButtons() > 0)
+  {
+    CLog::Log(LOGDEBUG, "%s - Seeking chapter is not allowed in menu set with buttons", __FUNCTION__);
+    return false;
+  }
+
+  bool enabled = IsSubtitleStreamEnabled();
+  int audio    = GetActiveAudioStream();
+  int subtitle = GetActiveSubtitleStream();
+
+  if (iChapter == (m_iPart + 1))
+  {
+    if (m_dll.dvdnav_next_pg_search(m_dvdnav) == DVDNAV_STATUS_ERR)
+    {
+      CLog::Log(LOGERROR, "dvdnav: dvdnav_next_pg_search( %s )", m_dll.dvdnav_err_to_string(m_dvdnav));
+      return false;
+    }
+  }
+  else
+  if (iChapter == (m_iPart - 1))
+  {
+    if (m_dll.dvdnav_prev_pg_search(m_dvdnav) == DVDNAV_STATUS_ERR)
+    {
+      CLog::Log(LOGERROR, "dvdnav: dvdnav_prev_pg_search( %s )", m_dll.dvdnav_err_to_string(m_dvdnav));
+      return false;
+    }
+  }
+  else  
   if (m_dll.dvdnav_part_play(m_dvdnav, m_iTitle, iChapter) == DVDNAV_STATUS_ERR)
   {
     CLog::Log(LOGERROR, "dvdnav: dvdnav_part_play failed( %s )", m_dll.dvdnav_err_to_string(m_dvdnav));
     return false;
   }
 
+  SetActiveSubtitleStream(subtitle);
+  SetActiveAudioStream(audio);
+  EnableSubtitleStream(enabled);
   return true;
 }
 
@@ -1239,9 +1265,6 @@ int CDVDInputStreamNavigator::ConvertAudioStreamId_ExternalToXBMC(int id)
     // non VTS_DOMAIN, only one stream is available
     return 0;
   }
-
-  CLog::Log(LOGWARNING, "%s - no stream found %d", __FUNCTION__, id);
-  return -1;
 }
 
 int CDVDInputStreamNavigator::ConvertSubtitleStreamId_XBMCToExternal(int id)
@@ -1313,7 +1336,4 @@ int CDVDInputStreamNavigator::ConvertSubtitleStreamId_ExternalToXBMC(int id)
     // non VTS_DOMAIN, only one stream is available
     return 0;
   }
-
-  CLog::Log(LOGWARNING, "%s - no stream found %d", __FUNCTION__, id);
-  return -1;
 }
