@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,20 +44,19 @@
 #include "music/dialogs/GUIDialogSongInfo.h"
 #include "addons/GUIDialogAddonInfo.h"
 #include "dialogs/GUIDialogSmartPlaylistEditor.h"
+#include "music/LastFmManager.h"
 #include "music/tags/MusicInfoTag.h"
 #include "guilib/GUIWindowManager.h"
-#include "guilib/Key.h"
 #include "dialogs/GUIDialogOK.h"
 #include "dialogs/GUIDialogYesNo.h"
 #include "guilib/GUIKeyboardFactory.h"
 #include "dialogs/GUIDialogProgress.h"
 #include "FileItem.h"
 #include "filesystem/File.h"
-#include "profiles/ProfilesManager.h"
 #include "storage/MediaManager.h"
-#include "settings/AdvancedSettings.h"
-#include "settings/MediaSettings.h"
 #include "settings/Settings.h"
+#include "settings/AdvancedSettings.h"
+#include "settings/GUISettings.h"
 #include "guilib/LocalizeStrings.h"
 #include "utils/TimeUtils.h"
 #include "utils/log.h"
@@ -66,7 +65,6 @@
 #include "utils/StringUtils.h"
 #include "URL.h"
 #include "music/infoscanner/MusicInfoScanner.h"
-#include "cores/IPlayer.h"
 
 using namespace std;
 using namespace XFILE;
@@ -83,7 +81,7 @@ using namespace MUSIC_INFO;
 CGUIWindowMusicBase::CGUIWindowMusicBase(int id, const CStdString &xmlFile)
     : CGUIMediaWindow(id, xmlFile)
 {
-  m_dlgProgress = NULL;
+
 }
 
 CGUIWindowMusicBase::~CGUIWindowMusicBase ()
@@ -145,10 +143,10 @@ bool CGUIWindowMusicBase::OnMessage(CGUIMessage& message)
 
       // save current window, unless the current window is the music playlist window
       if (GetID() != WINDOW_MUSIC_PLAYLIST &&
-          CSettings::Get().GetInt("mymusic.startwindow") != GetID())
+          g_settings.m_iMyMusicStartWindow != GetID())
       {
-        CSettings::Get().SetInt("mymusic.startwindow", GetID());
-        CSettings::Get().Save();
+        g_settings.m_iMyMusicStartWindow = GetID();
+        g_settings.Save();
       }
 
       return true;
@@ -174,11 +172,11 @@ bool CGUIWindowMusicBase::OnMessage(CGUIMessage& message)
         if (nWindow == GetID())
           return true;
 
-        CSettings::Get().SetInt("mymusic.startwindow", nWindow);
-        CSettings::Get().Save();
+        g_settings.m_iMyMusicStartWindow = nWindow;
+        g_settings.Save();
         g_windowManager.ChangeActiveWindow(nWindow);
 
-        CGUIMessage msg2(GUI_MSG_SETFOCUS, CSettings::Get().GetInt("mymusic.startwindow"), CONTROL_BTNTYPE);
+        CGUIMessage msg2(GUI_MSG_SETFOCUS, g_settings.m_iMyMusicStartWindow, CONTROL_BTNTYPE);
         g_windowManager.SendMessage(msg2);
 
         return true;
@@ -206,7 +204,7 @@ bool CGUIWindowMusicBase::OnMessage(CGUIMessage& message)
 
           // or be at the files window and have file deletion enabled
           else if (GetID() == WINDOW_MUSIC_FILES &&
-                   CSettings::Get().GetBool("filelists.allowfiledeletion"))
+                   g_guiSettings.GetBool("filelists.allowfiledeletion"))
           {
             OnDeleteItem(iItem);
           }
@@ -218,11 +216,11 @@ bool CGUIWindowMusicBase::OnMessage(CGUIMessage& message)
         else if (iAction == ACTION_PLAYER_PLAY)
         {
           // if playback is paused or playback speed != 1, return
-          if (g_application.m_pPlayer->IsPlayingAudio())
+          if (g_application.IsPlayingAudio())
           {
-            if (g_application.m_pPlayer->IsPausedPlayback())
+            if (g_application.m_pPlayer->IsPaused())
               return false;
-            if (g_application.m_pPlayer->GetPlaySpeed() != 1)
+            if (g_application.GetPlaySpeed() != 1)
               return false;
           }
 
@@ -233,7 +231,6 @@ bool CGUIWindowMusicBase::OnMessage(CGUIMessage& message)
         }
       }
     }
-    break;
   }
   return CGUIMediaWindow::OnMessage(message);
 }
@@ -253,7 +250,7 @@ bool CGUIWindowMusicBase::OnAction(const CAction &action)
   return CGUIMediaWindow::OnAction(action);
 }
 
-void CGUIWindowMusicBase::OnInfoAll(int iItem, bool bCurrent /* = false */, bool refresh /* = false */)
+void CGUIWindowMusicBase::OnInfoAll(int iItem, bool bCurrent, bool refresh)
 {
   CMusicDatabaseDirectory dir;
   CStdString strPath = m_vecItems->GetPath();
@@ -295,7 +292,7 @@ void CGUIWindowMusicBase::OnInfo(int iItem, bool bShowInfo)
 void CGUIWindowMusicBase::OnInfo(CFileItem *pItem, bool bShowInfo)
 {
   if ((pItem->IsMusicDb() && !pItem->HasMusicInfoTag()) || pItem->IsParentFolder() ||
-       URIUtils::IsSpecial(pItem->GetPath()) || StringUtils::StartsWithNoCase(pItem->GetPath(), "musicsearch://"))
+       URIUtils::IsSpecial(pItem->GetPath()) || pItem->GetPath().Left(14).Equals("musicsearch://"))
     return; // nothing to do
 
   if (!pItem->m_bIsFolder)
@@ -307,184 +304,289 @@ void CGUIWindowMusicBase::OnInfo(CFileItem *pItem, bool bShowInfo)
   // this function called from outside this window - make sure the database is open
   m_musicdatabase.Open();
 
+  CStdString strPath = pItem->GetPath();
+
+  // Try to find an album to lookup from the current item
+  CAlbum album;
+  CArtist artist;
+  bool foundAlbum = false;
+
+  album.idAlbum = -1;
+
   // we have a folder
   if (pItem->IsMusicDb())
   {
     CQueryParams params;
     CDirectoryNode::GetDatabaseInfo(pItem->GetPath(), params);
     if (params.GetAlbumId() == -1)
-      ShowArtistInfo(pItem);
+    { // artist lookup
+      artist.idArtist = params.GetArtistId();
+      artist.strArtist = StringUtils::Join(pItem->GetMusicInfoTag()->GetArtist(), g_advancedSettings.m_musicItemSeparator);
+    }
     else
-      ShowAlbumInfo(pItem);
+    { // album lookup
+      album.idAlbum = params.GetAlbumId();
+      album.strAlbum = pItem->GetMusicInfoTag()->GetAlbum();
+      album.artist = pItem->GetMusicInfoTag()->GetArtist();
+
+      // we're going to need it's path as well (we assume that there's only one) - this is for
+      // assigning thumbs to folders, and obtaining the local folder.jpg
+      m_musicdatabase.GetAlbumPath(album.idAlbum, strPath);
+    }
+  }
+  else
+  { // from filemode, so find the albums in the folder
+    CFileItemList items;
+    GetDirectory(strPath, items);
+
+    // show dialog box indicating we're searching the album name
+    if (m_dlgProgress && bShowInfo)
+    {
+      m_dlgProgress->SetHeading(185);
+      m_dlgProgress->SetLine(0, 501);
+      m_dlgProgress->SetLine(1, "");
+      m_dlgProgress->SetLine(2, "");
+      m_dlgProgress->StartModal();
+      m_dlgProgress->Progress();
+      if (m_dlgProgress->IsCanceled())
+      {
+        m_musicdatabase.Close();
+        return;
+      }
+    }
+    // check the first song we find in the folder, and grab it's album info
+    for (int i = 0; i < items.Size() && !foundAlbum; i++)
+    {
+      CFileItemPtr pItem = items[i];
+      pItem->LoadMusicTag();
+      if (pItem->HasMusicInfoTag() && pItem->GetMusicInfoTag()->Loaded() &&
+         !pItem->GetMusicInfoTag()->GetAlbum().IsEmpty())
+      {
+        // great, have a song - use it.
+        CSong song(*pItem->GetMusicInfoTag());
+        // this function won't be needed if/when the tag has idSong information
+        if (!m_musicdatabase.GetAlbumFromSong(song, album))
+        { // album isn't in the database - construct it from the tag info we have
+          CMusicInfoTag *tag = pItem->GetMusicInfoTag();
+          album.strAlbum = tag->GetAlbum();
+          album.artist = tag->GetAlbumArtist().empty() ? tag->GetArtist() : tag->GetAlbumArtist();
+          album.idAlbum = -1; // the -1 indicates it's not in the database
+        }
+        foundAlbum = true;
+      }
+    }
+    if (!foundAlbum)
+    {
+      CLog::Log(LOGINFO, "%s called on a folder containing no songs with tag info - nothing can be done", __FUNCTION__);
+      if (m_dlgProgress && bShowInfo)
+        m_dlgProgress->Close();
+      m_musicdatabase.Close();
+      return;
+    }
 
     if (m_dlgProgress && bShowInfo)
       m_dlgProgress->Close();
-    return;
   }
 
-  int albumID = m_musicdatabase.GetAlbumIdByPath(pItem->GetPath());
-  if (albumID != -1)
-  {
-    CAlbum album;
-    if (!m_musicdatabase.GetAlbum(albumID, album))
-      return;
-    CFileItem item(StringUtils::Format("musicdb://albums/%ld/", albumID), album);
-    if (ShowAlbumInfo(&item))
-      return;
-  }
-
-  CLog::Log(LOGINFO, "%s called on a folder containing no songs in the library - nothing can be done", __FUNCTION__);
+  if (album.idAlbum == -1 && foundAlbum == false)
+    ShowArtistInfo(artist, pItem->GetPath(), false, bShowInfo);
+  else
+    ShowAlbumInfo(album, strPath, false, bShowInfo);
+  m_musicdatabase.Close();
 }
 
-void CGUIWindowMusicBase::ShowArtistInfo(const CFileItem *pItem, bool bShowInfo /* = true */)
+void CGUIWindowMusicBase::OnManualAlbumInfo()
 {
-  CQueryParams params;
-  CDirectoryNode::GetDatabaseInfo(pItem->GetPath(), params);
-
-  ADDON::ScraperPtr scraper;
-  if (!m_musicdatabase.GetScraperForPath(pItem->GetPath(), scraper, ADDON::ADDON_SCRAPER_ARTISTS))
+  CAlbum album;
+  album.idAlbum = -1; // not in the db
+  if (!CGUIKeyboardFactory::ShowAndGetInput(album.strAlbum, g_localizeStrings.Get(16011), false))
     return;
 
-  CArtist artist;
-  if (!m_musicdatabase.GetArtist(params.GetArtistId(), artist))
-      return;
+  CStdString strArtist = StringUtils::Join(album.artist, g_advancedSettings.m_musicItemSeparator);
+  if (!CGUIKeyboardFactory::ShowAndGetInput(strArtist, g_localizeStrings.Get(16025), false))
+    return;
 
-  m_musicdatabase.GetArtistPath(params.GetArtistId(), artist.strPath);
-  while (1)
+  ShowAlbumInfo(album,"",true);
+}
+
+void CGUIWindowMusicBase::ShowArtistInfo(const CArtist& artist, const CStdString& path, bool bRefresh, bool bShowInfo)
+{
+  bool saveDb = artist.idArtist != -1;
+  if (!g_settings.GetCurrentProfile().canWriteDatabases() && !g_passwordManager.bMasterUser)
+    saveDb = false;
+
+  // check cache
+  CArtist artistInfo;
+  if (!bRefresh && m_musicdatabase.GetArtistInfo(artist.idArtist, artistInfo))
   {
-    // Check if we have the information in the database first
-    if (!m_musicdatabase.HasArtistBeenScraped(params.GetArtistId()))
-    {
-      if (!CProfilesManager::Get().GetCurrentProfile().canWriteDatabases() && !g_passwordManager.bMasterUser)
-        break; // should display a dialog saying no permissions
-
-      if (g_application.IsMusicScanning())
-      {
-        CGUIDialogOK::ShowAndGetInput(189, 14057, 0, 0);
-        break;
-      }
-
-      // show dialog box indicating we're searching the album
-      if (m_dlgProgress && bShowInfo)
-      {
-        m_dlgProgress->SetHeading(21889);
-        m_dlgProgress->SetLine(0, pItem->GetMusicInfoTag()->GetArtist());
-        m_dlgProgress->SetLine(1, "");
-        m_dlgProgress->SetLine(2, "");
-        m_dlgProgress->StartModal();
-      }
-
-      CMusicInfoScanner scanner;
-      if (scanner.UpdateDatabaseArtistInfo(artist, scraper, bShowInfo, m_dlgProgress) != INFO_ADDED)
-      {
-        CGUIDialogOK::ShowAndGetInput(21889, 0, 20199, 0);
-        break;
-      }
-    }
-
-    if (m_dlgProgress)
-      m_dlgProgress->Close();
+    if (!bShowInfo)
+      return;
 
     CGUIDialogMusicInfo *pDlgArtistInfo = (CGUIDialogMusicInfo*)g_windowManager.GetWindow(WINDOW_DIALOG_MUSIC_INFO);
     if (pDlgArtistInfo)
     {
-      pDlgArtistInfo->SetArtist(artist, artist.strPath);
-      pDlgArtistInfo->DoModal();
+      pDlgArtistInfo->SetArtist(artistInfo, path);
 
-      if (pDlgArtistInfo->NeedRefresh())
+      if (bShowInfo)
+        pDlgArtistInfo->DoModal();
+
+      if (!pDlgArtistInfo->NeedRefresh())
       {
-        m_musicdatabase.ClearArtistLastScrapedTime(params.GetArtistId());
-        continue;
-      } 
-      else if (pDlgArtistInfo->HasUpdatedThumb()) 
+        if (pDlgArtistInfo->HasUpdatedThumb())
+          Refresh();
+
+        return;
+      }
+      bRefresh = true;
+      m_musicdatabase.DeleteArtistInfo(artistInfo.idArtist);
+    }
+  }
+
+  // If we are scanning for music info in the background,
+  // other writing access to the database is prohibited.
+  if (g_application.IsMusicScanning())
+  {
+    CGUIDialogOK::ShowAndGetInput(189, 14057, 0, 0);
+    return;
+  }
+
+  CMusicArtistInfo info;
+  if (FindArtistInfo(artist.strArtist, info, bShowInfo ? (bRefresh ? SELECTION_FORCED : SELECTION_ALLOWED) : SELECTION_AUTO))
+  {
+    // download the album info
+    if ( info.Loaded() )
+    {
+      if (saveDb)
       {
-        Update(m_vecItems->GetPath());
+        // save to database
+        m_musicdatabase.SetArtistInfo(artist.idArtist, info.GetArtist());
+      }
+      if (m_dlgProgress && bShowInfo)
+        m_dlgProgress->Close();
+
+      // ok, show album info
+      CGUIDialogMusicInfo *pDlgArtistInfo = (CGUIDialogMusicInfo*)g_windowManager.GetWindow(WINDOW_DIALOG_MUSIC_INFO);
+      if (pDlgArtistInfo)
+      {
+        pDlgArtistInfo->SetArtist(info.GetArtist(), path);
+        if (bShowInfo)
+          pDlgArtistInfo->DoModal();
+
+        CArtist artistInfo = info.GetArtist();
+        artistInfo.idArtist = artist.idArtist;
+/*
+        if (pDlgAlbumInfo->HasUpdatedThumb())
+          UpdateThumb(artistInfo, path);
+*/
+        // just update for now
+        Refresh();
+        if (pDlgArtistInfo->NeedRefresh())
+        {
+          m_musicdatabase.DeleteArtistInfo(artistInfo.idArtist);
+          ShowArtistInfo(artist, path, true, bShowInfo);
+          return;
+        }
       }
     }
-    break;
+    else
+    {
+      // failed 2 download album info
+      CGUIDialogOK::ShowAndGetInput(21889, 0, 20199, 0);
+    }
   }
-  if (m_dlgProgress)
+
+  if (m_dlgProgress && bShowInfo)
     m_dlgProgress->Close();
 }
 
-bool CGUIWindowMusicBase::ShowAlbumInfo(const CFileItem *pItem, bool bShowInfo /* = true */)
+void CGUIWindowMusicBase::ShowAlbumInfo(const CAlbum& album, const CStdString& path, bool bRefresh, bool bShowInfo)
 {
-  CQueryParams params;
-  CDirectoryNode::GetDatabaseInfo(pItem->GetPath(), params);
+  bool saveDb = album.idAlbum != -1;
+  if (!g_settings.GetCurrentProfile().canWriteDatabases() && !g_passwordManager.bMasterUser)
+    saveDb = false;
 
-  ADDON::ScraperPtr scraper;
-  if (!m_musicdatabase.GetScraperForPath(pItem->GetPath(), scraper, ADDON::ADDON_SCRAPER_ALBUMS))
-    return false;
-
-  CAlbum album;
-  if (!m_musicdatabase.GetAlbum(params.GetAlbumId(), album))
-    return false;
-
-  m_musicdatabase.GetAlbumPath(params.GetAlbumId(), album.strPath);
-  while (1)
+  // check cache
+  CAlbum albumInfo;
+  if (!bRefresh && m_musicdatabase.GetAlbumInfo(album.idAlbum, albumInfo, &albumInfo.songs))
   {
-    if (!m_musicdatabase.HasAlbumBeenScraped(params.GetAlbumId()))
-    {
-      if (!CProfilesManager::Get().GetCurrentProfile().canWriteDatabases() && !g_passwordManager.bMasterUser)
-      {
-        // TODO: should display a dialog saying no permissions
-        if (m_dlgProgress)
-          m_dlgProgress->Close();
-        return false;
-      }
-
-      if (g_application.IsMusicScanning())
-      {
-        CGUIDialogOK::ShowAndGetInput(189, 14057, 0, 0);
-        if (m_dlgProgress)
-          m_dlgProgress->Close();
-        return false;
-      }
-
-      // show dialog box indicating we're searching the album
-      if (m_dlgProgress && bShowInfo)
-      {
-        m_dlgProgress->SetHeading(185);
-        m_dlgProgress->SetLine(0, pItem->GetMusicInfoTag()->GetAlbum());
-        m_dlgProgress->SetLine(1, StringUtils::Join(pItem->GetMusicInfoTag()->GetAlbumArtist(), g_advancedSettings.m_musicItemSeparator));
-        m_dlgProgress->SetLine(2, "");
-        m_dlgProgress->StartModal();
-      }
-
-      CMusicInfoScanner scanner;
-      if (scanner.UpdateDatabaseAlbumInfo(album, scraper, bShowInfo, m_dlgProgress) != INFO_ADDED)
-      {
-        CGUIDialogOK::ShowAndGetInput(185, 0, 500, 0);
-        if (m_dlgProgress)
-          m_dlgProgress->Close();
-        return false;
-      }
-    }
-
-    if (m_dlgProgress)
-      m_dlgProgress->Close();
+    if (!bShowInfo)
+      return;
 
     CGUIDialogMusicInfo *pDlgAlbumInfo = (CGUIDialogMusicInfo*)g_windowManager.GetWindow(WINDOW_DIALOG_MUSIC_INFO);
     if (pDlgAlbumInfo)
     {
-      pDlgAlbumInfo->SetAlbum(album, album.strPath);
-      pDlgAlbumInfo->DoModal();
+      pDlgAlbumInfo->SetAlbum(albumInfo, path);
+      if (bShowInfo)
+        pDlgAlbumInfo->DoModal();
 
-      if (pDlgAlbumInfo->NeedRefresh())
+      if (!pDlgAlbumInfo->NeedRefresh())
       {
-        m_musicdatabase.ClearAlbumLastScrapedTime(params.GetAlbumId());
-        continue;
+        if (pDlgAlbumInfo->HasUpdatedThumb())
+          UpdateThumb(albumInfo, path);
+        return;
       }
-      else if (pDlgAlbumInfo->HasUpdatedThumb())
+      bRefresh = true;
+      m_musicdatabase.DeleteAlbumInfo(albumInfo.idAlbum);
+    }
+  }
+
+  // If we are scanning for music info in the background,
+  // other writing access to the database is prohibited.
+  if (g_application.IsMusicScanning())
+  {
+    CGUIDialogOK::ShowAndGetInput(189, 14057, 0, 0);
+    return;
+  }
+
+  CMusicAlbumInfo info;
+  if (FindAlbumInfo(album.strAlbum, StringUtils::Join(album.artist, g_advancedSettings.m_musicItemSeparator), info, bShowInfo ? (bRefresh ? SELECTION_FORCED : SELECTION_ALLOWED) : SELECTION_AUTO))
+  {
+    // download the album info
+    if ( info.Loaded() )
+    {
+      // set album title from musicinfotag, not the one we got from allmusic.com
+      info.SetTitle(album.strAlbum);
+
+      if (saveDb)
       {
-        UpdateThumb(album, album.strPath);
+        // save to database
+        m_musicdatabase.SetAlbumInfo(album.idAlbum, info.GetAlbum(), info.GetSongs());
+      }
+      if (m_dlgProgress && bShowInfo)
+        m_dlgProgress->Close();
+
+      UpdateThumb(album, path);
+
+      // ok, show album info
+      CGUIDialogMusicInfo *pDlgAlbumInfo = (CGUIDialogMusicInfo*)g_windowManager.GetWindow(WINDOW_DIALOG_MUSIC_INFO);
+      if (pDlgAlbumInfo)
+      {
+        pDlgAlbumInfo->SetAlbum(info.GetAlbum(), path);
+        if (bShowInfo)
+          pDlgAlbumInfo->DoModal();
+
+        CAlbum albumInfo = info.GetAlbum();
+        albumInfo.idAlbum = album.idAlbum;
+        if (pDlgAlbumInfo->HasUpdatedThumb())
+          UpdateThumb(albumInfo, path);
+
+        if (pDlgAlbumInfo->NeedRefresh())
+        {
+          m_musicdatabase.DeleteAlbumInfo(albumInfo.idAlbum);
+          ShowAlbumInfo(album, path, true, bShowInfo);
+          return;
+        }
       }
     }
-    break;
+    else
+    {
+      // failed 2 download album info
+      CGUIDialogOK::ShowAndGetInput(185, 0, 500, 0);
+    }
   }
-  if (m_dlgProgress)
+
+  if (m_dlgProgress && bShowInfo)
     m_dlgProgress->Close();
-  return true;
 }
 
 void CGUIWindowMusicBase::ShowSongInfo(CFileItem* pItem)
@@ -558,7 +660,7 @@ void CGUIWindowMusicBase::OnQueueItem(int iItem)
   }
 
   g_playlistPlayer.Add(PLAYLIST_MUSIC, queuedItems);
-  if (g_playlistPlayer.GetPlaylist(PLAYLIST_MUSIC).size() && !g_application.m_pPlayer->IsPlayingAudio())
+  if (g_playlistPlayer.GetPlaylist(PLAYLIST_MUSIC).size() && !g_application.IsPlayingAudio())
   {
     if (m_guiState.get())
       m_guiState->SetPlaylistDirectory("playlistmusic://");
@@ -587,13 +689,11 @@ void CGUIWindowMusicBase::AddItemToPlayList(const CFileItemPtr &pItem, CFileItem
       // Genres will still require 2 lookups, and queuing the entire Genre folder
       // will require 3 lookups (genre, artist, album)
       CMusicDbUrl musicUrl;
-      if (musicUrl.FromString(pItem->GetPath()))
-      {
-        musicUrl.AppendPath("-1/");
-        CFileItemPtr item(new CFileItem(musicUrl.ToString(), true));
-        item->SetCanQueue(true); // workaround for CanQueue() check above
-        AddItemToPlayList(item, queuedItems);
-      }
+      musicUrl.FromString(pItem->GetPath());
+      musicUrl.AppendPath("-1/");
+      CFileItemPtr item(new CFileItem(musicUrl.ToString(), true));
+      item->SetCanQueue(true); // workaround for CanQueue() check above
+      AddItemToPlayList(item, queuedItems);
       return;
     }
   }
@@ -676,11 +776,108 @@ void CGUIWindowMusicBase::UpdateButtons()
   g_windowManager.SendMessage(msg2);
 
   // Select the current window as default item
-  CONTROL_SELECT_ITEM(CONTROL_BTNTYPE, CSettings::Get().GetInt("mymusic.startwindow") - WINDOW_MUSIC_FILES);
+  CONTROL_SELECT_ITEM(CONTROL_BTNTYPE, g_settings.m_iMyMusicStartWindow - WINDOW_MUSIC_FILES);
 
   CGUIMediaWindow::UpdateButtons();
 }
 
+bool CGUIWindowMusicBase::FindAlbumInfo(const CStdString& strAlbum, const CStdString& strArtist, CMusicAlbumInfo& album, ALLOW_SELECTION allowSelection)
+{
+  // show dialog box indicating we're searching the album
+  if (m_dlgProgress && allowSelection != SELECTION_AUTO)
+  {
+    m_dlgProgress->SetHeading(185);
+    m_dlgProgress->SetLine(0, strAlbum);
+    m_dlgProgress->SetLine(1, strArtist);
+    m_dlgProgress->SetLine(2, "");
+    m_dlgProgress->StartModal();
+  }
+
+  CMusicInfoScanner scanner;
+  CStdString strPath;
+  CStdString strTempAlbum(strAlbum);
+  CStdString strTempArtist(strArtist);
+  long idAlbum = m_musicdatabase.GetAlbumByName(strAlbum,strArtist);
+  strPath.Format("musicdb://3/%d/",idAlbum);
+
+  bool bCanceled(false);
+  bool needsRefresh(true);
+  do
+  {
+    if (!scanner.DownloadAlbumInfo(strPath,strTempArtist,strTempAlbum,bCanceled,album,m_dlgProgress))
+    {
+      if (bCanceled)
+        return false;
+      if (m_dlgProgress && allowSelection != SELECTION_AUTO)
+      {
+        if (!CGUIKeyboardFactory::ShowAndGetInput(strTempAlbum, g_localizeStrings.Get(16011), false))
+          return false;
+
+        if (!CGUIKeyboardFactory::ShowAndGetInput(strTempArtist, g_localizeStrings.Get(16025), false))
+          return false;
+      }
+      else
+        needsRefresh = false;
+    }
+    else
+      needsRefresh = false;
+  }
+  while (needsRefresh || bCanceled);
+
+  // Read the album information from the database if we are dealing with a DB album.
+  if (idAlbum != -1)
+    m_musicdatabase.GetAlbumInfo(idAlbum,album.GetAlbum(),&album.GetAlbum().songs);
+
+  album.SetLoaded(true);
+  return true;
+}
+
+bool CGUIWindowMusicBase::FindArtistInfo(const CStdString& strArtist, CMusicArtistInfo& artist, ALLOW_SELECTION allowSelection)
+{
+  // show dialog box indicating we're searching the album
+  if (m_dlgProgress && allowSelection != SELECTION_AUTO)
+  {
+    m_dlgProgress->SetHeading(21889);
+    m_dlgProgress->SetLine(0, strArtist);
+    m_dlgProgress->SetLine(1, "");
+    m_dlgProgress->SetLine(2, "");
+    m_dlgProgress->StartModal();
+  }
+
+  CMusicInfoScanner scanner;
+  CStdString strPath;
+  CStdString strTempArtist(strArtist);
+  long idArtist = m_musicdatabase.GetArtistByName(strArtist);
+  strPath.Format("musicdb://2/%u/",idArtist);
+
+  bool bCanceled(false);
+  bool needsRefresh(true);
+  do
+  {
+    if (!scanner.DownloadArtistInfo(strPath,strTempArtist,bCanceled,m_dlgProgress))
+    {
+      if (bCanceled)
+        return false;
+      if (m_dlgProgress && allowSelection != SELECTION_AUTO)
+      {
+        if (!CGUIKeyboardFactory::ShowAndGetInput(strTempArtist, g_localizeStrings.Get(16025), false))
+          return false;
+      }
+      else
+        needsRefresh = false;
+    }
+    else
+      needsRefresh = false;
+  }
+  while (needsRefresh || bCanceled);
+
+  if (!m_musicdatabase.GetArtistInfo(idArtist,artist.GetArtist()))
+    return false;
+
+  artist.SetLoaded();
+  return true;
+}
+
 void CGUIWindowMusicBase::GetContextButtons(int itemNumber, CContextButtons &buttons)
 {
   CFileItemPtr item;
@@ -693,7 +890,15 @@ void CGUIWindowMusicBase::GetContextButtons(int itemNumber, CContextButtons &but
     {
       if (!m_vecItems->IsPlugin() && (item->IsPlugin() || item->IsScript()))
         buttons.Add(CONTEXT_BUTTON_INFO,24003); // Add-on info
-      if (item->CanQueue() && !item->IsAddonsPath() && !item->IsScript())
+      if (item->GetExtraInfo().Equals("lastfmloved"))
+      {
+        buttons.Add(CONTEXT_BUTTON_LASTFM_UNLOVE_ITEM, 15295); //unlove
+      }
+      else if (item->GetExtraInfo().Equals("lastfmbanned"))
+      {
+        buttons.Add(CONTEXT_BUTTON_LASTFM_UNBAN_ITEM, 15296); //unban
+      }
+      else if (item->CanQueue() && !item->IsAddonsPath() && !item->IsScript())
       {
         buttons.Add(CONTEXT_BUTTON_QUEUE_ITEM, 13347); //queue
 
@@ -706,7 +911,7 @@ void CGUIWindowMusicBase::GetContextButtons(int itemNumber, CContextButtons &but
         else
         { // check what players we have, if we have multiple display play with option
           VECPLAYERCORES vecCores;
-          CPlayerCoreFactory::Get().GetPlayers(*item, vecCores);
+          CPlayerCoreFactory::GetPlayers(*item, vecCores);
           if (vecCores.size() >= 1)
             buttons.Add(CONTEXT_BUTTON_PLAY_WITH, 15213); // Play With...
         }
@@ -729,6 +934,8 @@ void CGUIWindowMusicBase::GetNonContextButtons(CContextButtons &buttons)
 {
   if (!m_vecItems->IsVirtualDirectoryRoot())
     buttons.Add(CONTEXT_BUTTON_GOTO_ROOT, 20128);
+  if (g_playlistPlayer.GetPlaylist(PLAYLIST_MUSIC).size() > 0)
+    buttons.Add(CONTEXT_BUTTON_NOW_PLAYING, 13350);
   buttons.Add(CONTEXT_BUTTON_SETTINGS, 5);
 }
 
@@ -778,8 +985,8 @@ bool CGUIWindowMusicBase::OnContextButton(int itemNumber, CONTEXT_BUTTON button)
   case CONTEXT_BUTTON_PLAY_WITH:
     {
       VECPLAYERCORES vecCores;  // base class?
-      CPlayerCoreFactory::Get().GetPlayers(*item, vecCores);
-      g_application.m_eForcedNextPlayer = CPlayerCoreFactory::Get().SelectPlayerDialog(vecCores);
+      CPlayerCoreFactory::GetPlayers(*item, vecCores);
+      g_application.m_eForcedNextPlayer = CPlayerCoreFactory::SelectPlayerDialog(vecCores);
       if( g_application.m_eForcedNextPlayer != EPC_NONE )
         OnClick(itemNumber);
       return true;
@@ -795,6 +1002,10 @@ bool CGUIWindowMusicBase::OnContextButton(int itemNumber, CONTEXT_BUTTON button)
       return true;
     }
 
+  case CONTEXT_BUTTON_NOW_PLAYING:
+    g_windowManager.ActivateWindow(WINDOW_MUSIC_PLAYLIST);
+    return true;
+
   case CONTEXT_BUTTON_GOTO_ROOT:
     Update("");
     return true;
@@ -802,6 +1013,20 @@ bool CGUIWindowMusicBase::OnContextButton(int itemNumber, CONTEXT_BUTTON button)
   case CONTEXT_BUTTON_SETTINGS:
     g_windowManager.ActivateWindow(WINDOW_SETTINGS_MYMUSIC);
     return true;
+  case CONTEXT_BUTTON_LASTFM_UNBAN_ITEM:
+    if (CLastFmManager::GetInstance()->Unban(*item->GetMusicInfoTag()))
+    {
+      g_directoryCache.ClearDirectory(m_vecItems->GetPath());
+      Refresh(true);
+    }
+    return true;
+  case CONTEXT_BUTTON_LASTFM_UNLOVE_ITEM:
+    if (CLastFmManager::GetInstance()->Unlove(*item->GetMusicInfoTag()))
+    {
+      g_directoryCache.ClearDirectory(m_vecItems->GetPath());
+      Refresh(true);
+    }
+    return true;
   default:
     break;
   }
@@ -948,7 +1173,7 @@ bool CGUIWindowMusicBase::OnPlayMedia(int iItem)
   CFileItemPtr pItem = m_vecItems->Get(iItem);
 
   // party mode
-  if (g_partyModeManager.IsEnabled())
+  if (g_partyModeManager.IsEnabled() && !pItem->IsLastFM())
   {
     CPlayList playlistTemp;
     playlistTemp.Add(pItem);
@@ -960,7 +1185,7 @@ bool CGUIWindowMusicBase::OnPlayMedia(int iItem)
     // turned on, but we still want to use the playlist player in order to handle more queued items
     // following etc.
     // Karaoke items also can be added in runtime (while the song is played), so it should be queued too.
-    if ( (CSettings::Get().GetBool("musicplayer.queuebydefault") && g_windowManager.GetActiveWindow() != WINDOW_MUSIC_PLAYLIST_EDITOR)
+    if ( (g_guiSettings.GetBool("musicplayer.queuebydefault") && g_windowManager.GetActiveWindow() != WINDOW_MUSIC_PLAYLIST_EDITOR)
        || pItem->IsKaraoke() )
     {
       // TODO: Should the playlist be cleared if nothing is already playing?
@@ -982,13 +1207,13 @@ void CGUIWindowMusicBase::UpdateThumb(const CAlbum &album, const CStdString &pat
   // check user permissions
   bool saveDb = album.idAlbum != -1;
   bool saveDirThumb = true;
-  if (!CProfilesManager::Get().GetCurrentProfile().canWriteDatabases() && !g_passwordManager.bMasterUser)
+  if (!g_settings.GetCurrentProfile().canWriteDatabases() && !g_passwordManager.bMasterUser)
   {
     saveDb = false;
     saveDirThumb = false;
   }
 
-  CStdString albumThumb = m_musicdatabase.GetArtForItem(album.idAlbum, MediaTypeAlbum, "thumb");
+  CStdString albumThumb = m_musicdatabase.GetArtForItem(album.idAlbum, "album", "thumb");
 
   // Update the thumb in the music database (songs + albums)
   CStdString albumPath(path);
@@ -998,7 +1223,7 @@ void CGUIWindowMusicBase::UpdateThumb(const CAlbum &album, const CStdString &pat
   // Update currently playing song if it's from the same album.  This is necessary as when the album
   // first gets it's cover, the info manager's item doesn't have the updated information (so will be
   // sending a blank thumb to the skin.)
-  if (g_application.m_pPlayer->IsPlayingAudio())
+  if (g_application.IsPlayingAudio())
   {
     const CMusicInfoTag* tag=g_infoManager.GetCurrentSongTag();
     if (tag)
@@ -1017,17 +1242,26 @@ void CGUIWindowMusicBase::UpdateThumb(const CAlbum &album, const CStdString &pat
   // Save this thumb as the directory thumb if it's the only album in the folder (files view nicety)
   // We do this by grabbing all the songs in the folder, and checking to see whether they come
   // from the same album.
-  if (saveDirThumb && CFile::Exists(albumThumb) && !albumPath.empty() && !URIUtils::IsCDDA(albumPath))
+  if (saveDirThumb && CFile::Exists(albumThumb) && !albumPath.IsEmpty() && !URIUtils::IsCDDA(albumPath))
   {
     CFileItemList items;
     GetDirectory(albumPath, items);
     OnRetrieveMusicInfo(items);
+    VECSONGS songs;
+    for (int i = 0; i < items.Size(); i++)
+    {
+      CFileItemPtr item = items[i];
+      if (item->HasMusicInfoTag() && item->GetMusicInfoTag()->Loaded())
+      {
+        CSong song(*item->GetMusicInfoTag());
+        songs.push_back(song);
+      }
+    }
     VECALBUMS albums;
-    CMusicInfoScanner::FileItemsToAlbums(items, albums);
+    CMusicInfoScanner::CategoriseAlbums(songs, albums);
     if (albums.size() == 1)
     { // set as folder thumb as well
-      CMusicThumbLoader loader;
-      loader.SetCachedImage(items, "thumb", albumPath);
+      CThumbLoader::SetCachedImage(items, "thumb", albumPath);
     }
   }
 
@@ -1045,7 +1279,7 @@ void CGUIWindowMusicBase::UpdateThumb(const CAlbum &album, const CStdString &pat
 void CGUIWindowMusicBase::OnRetrieveMusicInfo(CFileItemList& items)
 {
   if (items.GetFolderCount()==items.Size() || items.IsMusicDb() ||
-     (!CSettings::Get().GetBool("musicfiles.usetags") && !items.IsCDDA()))
+     (!g_guiSettings.GetBool("musicfiles.usetags") && !items.IsCDDA()))
   {
     return;
   }
@@ -1094,15 +1328,12 @@ bool CGUIWindowMusicBase::GetDirectory(const CStdString &strDirectory, CFileItem
   items.SetArt("thumb", "");
   bool bResult = CGUIMediaWindow::GetDirectory(strDirectory, items);
   if (bResult)
-  {
-    CMusicThumbLoader loader;
-    loader.FillThumb(items);
-  }
+    CMusicThumbLoader::FillThumb(items);
 
   // add in the "New Playlist" item if we're in the playlists folder
   if ((items.GetPath() == "special://musicplaylists/") && !items.Contains("newplaylist://"))
   {
-    CFileItemPtr newPlaylist(new CFileItem(CProfilesManager::Get().GetUserDataItem("PartyMode.xsp"),false));
+    CFileItemPtr newPlaylist(new CFileItem(g_settings.GetUserDataItem("PartyMode.xsp"),false));
     newPlaylist->SetLabel(g_localizeStrings.Get(16035));
     newPlaylist->SetLabelPreformated(true);
     newPlaylist->m_bIsFolder = true;
@@ -1126,6 +1357,10 @@ bool CGUIWindowMusicBase::GetDirectory(const CStdString &strDirectory, CFileItem
   return bResult;
 }
 
+void CGUIWindowMusicBase::OnPrepareFileItems(CFileItemList &items)
+{
+}
+
 bool CGUIWindowMusicBase::CheckFilterAdvanced(CFileItemList &items) const
 {
   CStdString content = items.GetContent();
@@ -1138,26 +1373,21 @@ bool CGUIWindowMusicBase::CheckFilterAdvanced(CFileItemList &items) const
 
 bool CGUIWindowMusicBase::CanContainFilter(const CStdString &strDirectory) const
 {
-  return StringUtils::StartsWithNoCase(strDirectory, "musicdb://");
+  return StringUtils::StartsWith(strDirectory, "musicdb://");
 }
 
 void CGUIWindowMusicBase::OnInitWindow()
 {
   CGUIMediaWindow::OnInitWindow();
-  if (CMediaSettings::Get().GetMusicNeedsUpdate() == 35 && !g_application.IsMusicScanning() &&
+  if (g_settings.m_musicNeedsUpdate == 27 && !g_application.IsMusicScanning() &&
       g_infoManager.GetLibraryBool(LIBRARY_HAS_MUSIC))
   {
     // rescan of music library required
     if (CGUIDialogYesNo::ShowAndGetInput(799, 800, 801, -1))
     {
-      int flags = CMusicInfoScanner::SCAN_RESCAN;
-      if (CSettings::Get().GetBool("musiclibrary.downloadinfo"))
-        flags |= CMusicInfoScanner::SCAN_ONLINE;
-      if (CSettings::Get().GetBool("musiclibrary.backgroundupdate"))
-        flags |= CMusicInfoScanner::SCAN_BACKGROUND;
-      g_application.StartMusicScan("", flags);
-      CMediaSettings::Get().SetMusicNeedsUpdate(0); // once is enough (user may interrupt, but that's up to them)
-      CSettings::Get().Save();
+      g_application.StartMusicScan("", CMusicInfoScanner::SCAN_RESCAN);
+      g_settings.m_musicNeedsUpdate = 0; // once is enough (user may interrupt, but that's up to them)
+      g_settings.Save();
     }
   }
 }
