@@ -1,7 +1,7 @@
 #pragma once
 
 /*
- *      Copyright (C) 2005-2008 Team XBMC
+ *      Copyright (C) 2005-2012 Team XBMC
  *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
@@ -15,9 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
- *  http://www.gnu.org/copyleft/gpl.html
+ *  along with XBMC; see the file COPYING.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -35,6 +34,7 @@
 class CSysData
 {
 public:
+  enum INTERNET_STATE { UNKNOWN, CONNECTED, NO_DNS, DISCONNECTED };
   CSysData()
   {
     Reset();
@@ -42,18 +42,17 @@ public:
 
   void Reset()
   {
-    haveInternetState = false;
-    internetState = "";
+    internetState = UNKNOWN;
   };
 
-  bool haveInternetState;
   CStdString systemUptime;
   CStdString systemTotalUptime;
-  CStdString internetState;
+  INTERNET_STATE internetState;
   CStdString videoEncoder;
   CStdString cpuFrequency;
   CStdString kernelVersion;
   CStdString macAddress;
+  CStdString batteryLevel;
 };
 
 class CSysInfoJob : public CJob
@@ -64,14 +63,15 @@ public:
   virtual bool DoWork();
   const CSysData &GetData() const;
 
+  static CSysData::INTERNET_STATE GetInternetState();
 private:
   bool SystemUpTime(int iInputMinutes, int &iMinutes, int &iHours, int &iDays);
   double GetCPUFrequency();
-  CStdString GetInternetState();
   CStdString GetSystemUpTime(bool bTotalUptime);
   CStdString GetCPUFreqInfo();
   CStdString GetMACAddress();
   CStdString GetVideoEncoder();
+  CStdString GetBatteryLevel();
 
   CSysData m_info;
 };
@@ -79,6 +79,17 @@ private:
 class CSysInfo : public CInfoLoader
 {
 public:
+  enum WindowsVersion
+  {
+    WindowsVersionUnknown = -1, // Undetected, unsupported Windows version or OS in not Windows
+    WindowsVersionWinXP,        // Windows XP, Windows Server 2003 (R2), Windows Home Server
+    WindowsVersionVista,        // Windows Vista, Windows Server 2008
+    WindowsVersionWin7,         // Windows 7, Windows Server 2008 R2
+    WindowsVersionWin8,         // Windows 8, Windows Server 2012
+    /* Insert new Windows versions here, when they'll be known */
+    WindowsVersionFuture = 100  // Future Windows version, not known to code
+  };
+
   CSysInfo(void);
   virtual ~CSysInfo();
 
@@ -88,22 +99,32 @@ public:
   bool GetHDDInfo(CStdString& strHDDModel, CStdString& strHDDSerial,CStdString& strHDDFirmware,CStdString& strHDDpw,CStdString& strHDDLockState);
   bool GetRefurbInfo(CStdString& rfi_FirstBootTime, CStdString& rfi_PowerCycleCount);
 
-  bool CreateBiosBackup();
-  bool CreateEEPROMBackup();
-  void WriteTXTInfoFile();
-
-#if defined(_LINUX) && !defined(__APPLE__)
+#if defined(_LINUX) && !defined(TARGET_DARWIN) && !defined(__FreeBSD__)
   CStdString GetLinuxDistro();
 #endif
 #ifdef _LINUX
   CStdString GetUnameVersion();
-#endif    
+#endif
+#if defined(TARGET_WINDOWS)
+  CStdString CSysInfo::GetUAWindowsVersion();
+#endif
   CStdString GetUserAgent();
-  bool HasInternet() const;
-  bool IsAppleTV();
+  bool HasInternet();
+  bool IsAppleTV2();
+  bool HasVDADecoder();
+  bool HasVideoToolBoxDecoder();
   bool IsAeroDisabled();
+  bool IsVistaOrHigher();
+  static bool IsWindowsVersion(WindowsVersion ver);
+  static bool IsWindowsVersionAtLeast(WindowsVersion ver);
+  static WindowsVersion GetWindowsVersion();
+  static bool IsOS64bit();
   static CStdString GetKernelVersion();
-  CStdString GetXBVerInfo();
+  CStdString GetCPUModel();
+  CStdString GetCPUBogoMips();
+  CStdString GetCPUHardware();
+  CStdString GetCPURevision();
+  CStdString GetCPUSerial();
   bool GetDiskSpace(const CStdString drive,int& iTotal, int& iTotalFree, int& iTotalUsed, int& iPercentFree, int& iPercentUsed);
   CStdString GetHddSpaceInfo(int& percent, int drive, bool shortText=false);
   CStdString GetHddSpaceInfo(int drive, bool shortText=false);
@@ -115,6 +136,7 @@ protected:
 
 private:
   CSysData m_info;
+  static WindowsVersion m_WinVer;
   void Reset();
 };
 
