@@ -36,6 +36,7 @@
 #include "osx/XBMCHelper.h"
 #include "utils/SystemInfo.h"
 #include "osx/CocoaInterface.h"
+#include "osx/DarwinUtils.h"
 #undef BOOL
 
 #import <SDL/SDL_video.h>
@@ -1359,8 +1360,9 @@ bool CWinSystemOSX::IsObscured(void)
         // if the windowBounds completely encloses our bounds, we are obscured.
         if (!obscureLogged)
         {
-          const char* cstr = CFStringGetCStringPtr(ownerName, CFStringGetSystemEncoding());
-          CLog::Log(LOGDEBUG, "WinSystemOSX: Fullscreen window %s obscures XBMC!", cstr);
+          std::string appName;
+          if (DarwinCFStringRefToUTF8String(ownerName, appName))
+            CLog::Log(LOGDEBUG, "WinSystemOSX: Fullscreen window %s obscures XBMC!", appName.c_str());
           obscureLogged = true;
         }
         m_obscured = true;
